@@ -17,6 +17,7 @@ from __future__ import annotations
 
 import json
 import logging
+import math
 from typing import List  # @UnusedImport
 from typing import (
     TYPE_CHECKING,
@@ -41,8 +42,7 @@ from pde.tools.cuboid import Cuboid
 from pde.tools.docstrings import fill_in_docstring
 from pde.tools.output import display_progress
 from pde.tools.plotting import PlotReference, plot_on_axes
-from pde.trackers.base import InfoDict
-from pde.trackers.intervals import IntervalType
+from pde.trackers.base import InfoDict, IntervalData
 
 from .droplets import SphericalDroplet, droplet_from_data
 
@@ -479,10 +479,10 @@ class Emulsion(list):
         if len(self) == 0:
             return {
                 "count": 0,
-                "radius_mean": np.nan,
-                "radius_std": np.nan,
-                "volume_mean": np.nan,
-                "volume_std": np.nan,
+                "radius_mean": math.nan,
+                "radius_std": math.nan,
+                "volume_mean": math.nan,
+                "volume_std": math.nan,
             }
 
         radii = [droplet.radius for droplet in self]
@@ -845,9 +845,7 @@ class EmulsionTimeCourse:
 
     @fill_in_docstring
     def tracker(
-        self,
-        interval: Union[int, float, IntervalType] = 1,
-        filename: Optional[str] = None,
+        self, interval: IntervalData = 1, filename: Optional[str] = None
     ) -> "DropletTracker":
         """return a tracker that analyzes emulsions during simulations
 
