@@ -14,6 +14,7 @@ Functions for analyzing phase field images of emulsions.
 """
 
 import logging
+import math
 import warnings
 from functools import reduce
 from typing import Any, Dict, Optional, Sequence, Tuple, Union
@@ -47,6 +48,43 @@ from .droplets import (
 from .emulsions import Emulsion
 
 
+def threshold_otsu(data: np.ndarray, nbins: int = 256) -> float:
+    """Find the threshold value for a bimodal histogram using the Otsu method.
+
+    If you have a distribution that is bimodal, i.e., with two peaks and a valley
+    between them, then you can use this to find the location of that valley, which
+    splits the distribution into two.
+
+    Args:
+        data (:class:`~numpy.ndarray`):
+            The data to be analyzed
+        nbins (int):
+            The number of bins in the histogram, which defines the accuracy of the
+            determined threshold.
+
+    Modified from https://stackoverflow.com/a/71345917/932593, which is based on the
+    the SciKit Image threshold_otsu implementation:
+    https://github.com/scikit-image/scikit-image/blob/70fa904eee9ef370c824427798302551df57afa1/skimage/filters/thresholding.py#L312
+    """
+    counts, bin_edges = np.histogram(data.flat, bins=nbins)
+    bin_centers = (bin_edges[1:] + bin_edges[:-1]) / 2
+
+    # class probabilities for all possible thresholds
+    weight1 = np.cumsum(counts)
+    weight2 = np.cumsum(counts[::-1])[::-1]
+    # class means for all possible thresholds
+    mean1 = np.cumsum(counts * bin_centers) / weight1
+    mean2 = (np.cumsum((counts * bin_centers)[::-1]) / weight2[::-1])[::-1]
+
+    # Clip ends to align class 1 and class 2 variables:
+    # The last value of ``weight1``/``mean1`` should pair with zero values in
+    # ``weight2``/``mean2``, which do not exist.
+    variance12 = weight1[:-1] * weight2[1:] * (mean1[:-1] - mean2[1:]) ** 2
+
+    idx = np.argmax(variance12)
+    return float(bin_centers[idx])
+
+
 def _locate_droplets_in_mask_cartesian(
     grid: CartesianGrid, mask: np.ndarray
 ) -> Emulsion:
@@ -269,10 +307,12 @@ def locate_droplets_in_mask(grid: GridBase, mask: np.ndarray) -> Emulsion:
 def locate_droplets(
     phase_field: ScalarField,
     threshold: Union[float, str] = 0.5,
-    modes: int = 0,
+    *,
     minimal_radius: float = 0,
-    refine: bool = False,
+    modes: int = 0,
     interface_width: Optional[float] = None,
+    refine: bool = False,
+    refine_args: Dict[str, Any] = None,
 ) -> Emulsion:
     """Locates droplets in the phase field
 
@@ -284,22 +324,31 @@ def locate_droplets(
         phase_field (:class:`~pde.fields.ScalarField`):
             Scalar field that describes the concentration field of droplets
         threshold (float or str):
-            The threshold for binarizing the image. The special value 'auto'
-            takes the mean between the minimum and the maximum of the data as a
-            guess.
-        modes (int):
-            The number of perturbation modes that should be included.
-            If `modes=0`, droplets are assumed to be spherical. Note that the
-            mode amplitudes are only determined when `refine=True`.
+            The threshold for binarizing the image. If a value is given it is used
+            directly. Otherwise, the following algorithms are supported:
+
+            * `extrema`: take mean between the minimum and the maximum of the data
+            * `mean`: take the mean over the entire data
+            * `otsu`: use Otsu's method implemented in :func:`threshold_otsu`
+
+            The special value `auto` currently defaults to the `extrema` method.
+
         minimal_radius (float):
-            The smallest radius of droplets to include in the list. This can be
-            used to filter out fluctuations in noisy simulations.
-        refine (bool):
-            Flag determining whether the droplet properties should be refined
-            using fitting. This is a potentially slow procedure.
+            The smallest radius of droplets to include in the list. This can be used to
+            filter out fluctuations in noisy simulations.
+        modes (int):
+            The number of perturbation modes that should be included. If `modes=0`,
+            droplets are assumed to be spherical. Note that the mode amplitudes are only
+            determined when `refine=True`.
         interface_width (float, optional):
-            Interface width of the located droplets, which is also used as a
-            starting value for the fitting if droplets are refined.
+            Interface width of the located droplets, which is also used as a starting
+            value for the fitting if droplets are refined.
+        refine (bool):
+            Flag determining whether the droplet properties should be refined using
+            fitting. This is a potentially slow procedure.
+        refine_args (dict):
+            Additional keyword arguments passed on to :func:`refine_droplet`. Only has
+            an effect if `refine=True`.
 
     Returns:
         :class:`~droplets.emulsions.Emulsion`: All detected droplets
@@ -309,10 +358,16 @@ def locate_droplets(
 
     if modes > 0 and dim not in [2, 3]:
         raise ValueError("Perturbed droplets only supported for 2d and 3d")
+    if refine_args is None:
+        refine_args = {}
 
     # determine actual threshold
-    if threshold == "auto":
+    if threshold == "extrema" or threshold == "auto":
         threshold = float(phase_field.data.min() + phase_field.data.max()) / 2
+    elif threshold == "mean":
+        threshold = float(phase_field.data.mean())
+    elif threshold == "otsu":
+        threshold = threshold_otsu(phase_field.data)
     else:
         threshold = float(threshold)
 
@@ -351,7 +406,7 @@ def locate_droplets(
         # refine droplets if necessary
         if refine:
             try:
-                droplet = refine_droplet(phase_field, droplet)
+                droplet = refine_droplet(phase_field, droplet, **refine_args)
             except ValueError:
                 continue  # do not add the droplet to the list
         droplets.append(droplet)
@@ -366,6 +421,10 @@ def locate_droplets(
 def refine_droplet(
     phase_field: ScalarField,
     droplet: DiffuseDroplet,
+    *,
+    vmin: float = 0.0,
+    vmax: float = 1.0,
+    adjust_values: bool = False,
     least_squares_params: Optional[Dict[str, Any]] = None,
 ) -> DiffuseDroplet:
     """Refines droplet parameters by fitting to phase field
@@ -381,9 +440,19 @@ def refine_droplet(
         droplet (:class:`~droplets.droplets.SphericalDroplet`):
             Droplet that should be refined. This could also be a subclass of
             :class:`SphericalDroplet`
+        vmin (float):
+            The intensity value of the dilute phase surrounding the droplet. If `None`,
+            the value will be determined automatically.
+        vmax (float):
+            The intensity value inside the droplet. If `None`, the value will be
+            determined automatically.
+        adjust_value (bool):
+            Flag determining whether the intensity values will be included in the
+            fitting procedure. The default value `False` implies that the intensity
+            values are regarded fixed.
         least_squares_params (dict):
-            Dictionary of parameters that influence the fitting; see the
-            documentation of scipy.optimize.least_squares
+            Dictionary of parameters that influence the fitting; see the documentation
+            of :func:`scipy.optimize.least_squares`.
 
     Returns:
         :class:`droplets.droplets.DiffuseDroplet`: The refined droplet as an instance of
@@ -416,20 +485,53 @@ def refine_droplet(
     l, h = droplet.data_bounds
     bounds = l[free], h[free]
 
-    def _image_deviation(params):
-        """helper function evaluating the residuals"""
-        # generate the droplet
-        data_flat[free] = params
-        droplet.data = unstructured_to_structured(data_flat, dtype=dtype)
-        droplet.check_data()
-        img = droplet._get_phase_field(phase_field.grid)[mask]
-        return img - data_mask
+    # determine the intensities outside and inside the droplet
+    if vmin is None:
+        vmin = np.min(data_mask)
+    if vmax is None:
+        vmax = np.max(data_mask)
+    vrng = vmax - vmin
 
-    # do the least square optimization
-    result = optimize.least_squares(
-        _image_deviation, data_flat[free], bounds=bounds, **least_squares_params
-    )
-    data_flat[free] = result.x
+    if adjust_values:
+        # fit intensities in addition to all droplet parameters
+
+        # add vmin and vrng as separate fitting parameters
+        parameters = np.r_[data_flat[free], vmin, vmax]
+        bounds = np.r_[bounds[0], vmin - vrng, 0], np.r_[bounds[1], vmax, 3 * vrng]
+
+        def _image_deviation(params):
+            """helper function evaluating the residuals"""
+            # generate the droplet
+            data_flat[free] = params[:-2]
+            vmin, vrng = params[-2:]
+            droplet.data = unstructured_to_structured(data_flat, dtype=dtype)
+            droplet.check_data()
+            img = vmin + vrng * droplet._get_phase_field(phase_field.grid)[mask]
+            return img - data_mask
+
+        # do the least square optimization
+        result = optimize.least_squares(
+            _image_deviation, parameters, bounds=bounds, **least_squares_params
+        )
+        data_flat[free] = result.x[:-2]
+
+    else:
+        # fit only droplet parameters and assume all intensities fixed
+
+        def _image_deviation(params):
+            """helper function evaluating the residuals"""
+            # generate the droplet
+            data_flat[free] = params
+            droplet.data = unstructured_to_structured(data_flat, dtype=dtype)
+            droplet.check_data()
+            img = vmin + vrng * droplet._get_phase_field(phase_field.grid)[mask]
+            return img - data_mask
+
+        # do the least square optimization
+        result = optimize.least_squares(
+            _image_deviation, data_flat[free], bounds=bounds, **least_squares_params
+        )
+        data_flat[free] = result.x
     droplet.data = unstructured_to_structured(data_flat, dtype=dtype)
 
     # normalize the droplet position
@@ -589,7 +691,7 @@ def get_length_scale(
 
     elif method == "structure_factor_maximum" or method == "structure_factor_peak":
         # calculate the structure factor
-        k_mag, sf = get_structure_factor(scalar_field, smoothing=None)
+        k_mag, sf = get_structure_factor(scalar_field, smoothing=None, add_zero=True)
 
         # smooth the structure factor
         if kwargs.pop("smoothing", None) is None:
@@ -600,7 +702,8 @@ def get_length_scale(
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
 
-            max_est = k_mag[np.argmax(sf)]
+            # determine maximum (excluding k=0)
+            max_est = k_mag[1 + np.argmax(sf[1:])]
             bracket = np.array([0.2, 1, 5]) * max_est
             logger.debug(f"Search maximum of structure factor in interval {bracket}")
             try:
@@ -609,7 +712,7 @@ def get_length_scale(
                 )
             except Exception:
                 logger.exception("Could not determine maximal structure factor")
-                length_scale = np.nan
+                length_scale = math.nan
             else:
                 if not result.success:
                     logger.warning(
@@ -624,12 +727,12 @@ def get_length_scale(
     elif method == "droplet_detection":
         # calculate the length scale from detected droplets
         droplets = locate_droplets(scalar_field, **kwargs)
-        kwargs = None  # clear kwargs, so no warning is raised
+        kwargs = {}  # clear kwargs, so no warning is raised
 
         # get the axes along which droplets can be placed
         grid = scalar_field.grid
         axes = set(range(grid.dim)) - set(grid.coordinate_constraints)
-        volume = 1
+        volume = 1.0
         for ax in axes:
             volume *= grid.axes_bounds[ax][1] - grid.axes_bounds[ax][0]
 
@@ -651,6 +754,7 @@ def get_length_scale(
 
 
 __all__ = [
+    "threshold_otsu",
     "locate_droplets",
     "refine_droplet",
     "get_structure_factor",
